--[[
Name: PetTimeToLive
Revision: $Revision: 10 $
Author(s): mitch0
Website: http://www.wowace.com/projects/pet-time-to-live
SVN: svn://svn.wowace.com/wow/pet-time-to-live/maguiInline/trunk
License: Public Domain
]]

local AppName = "PetTimeToLive"
local VERSION = AppName .. "-r" .. ("$Revision: 10 $"):match("%d+")

local LSM = LibStub:GetLibrary("LibSharedMedia-3.0", true)
local L = LibStub("AceLocale-3.0"):GetLocale(AppName)

-- cached stuff

local GetPetTimeRemaining = GetPetTimeRemaining

-- hard-coded config stuff

local DefaultBGTexture = "Blizzard Dialog Background"
local DefaultBGFile = [[Interface\DialogFrame\UI-DialogBox-Background]]
local DefaultEdgeTexture = "Blizzard Tooltip"
local DefaultEdgeFile = [[Interface\Tooltips\UI-Tooltip-Border]]
local DefaultFontName = "Friz Quadrata TT"
local DefaultFontPath = GameFontNormal:GetFont()
local DefaultFrameWidth = 48
local DefaultFrameHeight = 24

local PtrScale = 1000
local MaxPtr = 10 * 60 * PtrScale
local NormalUpdateDelay = 1.0/10 -- update frequency == 1/NormalUpdateDelay
local WarningUpdateDelay = 1.0/25 -- update frequency while warning == 1/WarningUpdateDelay, must be <= NormalUpdateDelay

-- internal vars

local _ -- throwaway
local db
local lastUpdate = 0 -- time since last real update
local updateDelay = NormalUpdateDelay

---------------------------------

PetTimeToLive = LibStub("AceAddon-3.0"):NewAddon(AppName, "AceConsole-3.0", "AceEvent-3.0")
local pttl = PetTimeToLive

PetTimeToLive:SetDefaultModuleState(false)

PetTimeToLive.version = VERSION
PetTimeToLive.AppName = AppName

-- Default DB stuff

local function makeColor(r, g, b, a)
    a = a or 1.0
    return { ["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a }
end

local defaults = {
    profile = {
        locked = false,
        minimap = {},

        point = "CENTER",
        relPoint = "CENTER",
        x = 0,
        y = 50,

        color = makeColor(1.0, 0.82, 0),
        warnColor = makeColor(0.9, 0.065, 0.075),
        warnThreshold = 5.0,
        warnSound = false,
        warnSoundName = nil,

        font = DefaultFontName,
        fontSize = 16,
        fontOutline = "",
        strata = "HIGH",

        frameWidth = DefaultFrameWidth,
        frameHeight = DefaultFrameHeight,

        bgEnabled = true,
        bgTexture = DefaultBGTexture,
        bgBorderTexture = DefaultEdgeTexture,
        bgTile = false,
        bgTileSize = 32,
        bgEdgeSize = 16,
        bgColor = makeColor(1, 1, 1),
        bgBorderColor = makeColor(0.8, 0.6, 0.0),
    },
}

function PetTimeToLive:setTextColor(color)
    self.text:SetTextColor(color.r, color.g, color.b, color.a)
end

function PetTimeToLive:start()
    self.mainFrame:Show()
    self:setTextColor(db.color)
    self:update()
end

function PetTimeToLive:stop()
    self.warned = nil
    self.text:SetText(nil)
    updateDelay = NormalUpdateDelay
    if (db.locked) then
        self.mainFrame:Hide()
    end
end

function PetTimeToLive:update()
    local ptr = GetPetTimeRemaining()
    if (not ptr or ptr >= MaxPtr) then
        -- ptr gets HUGE just before despawning the pet, probably a bug
        -- might need to change this logic if we want to support pets with longer than 10 min lifetime
        self:stop()
        return
    end
    ptr = ptr / PtrScale
    if (ptr > 90) then
        self.text:SetText(string.format("%d:%02d", ptr / 60, ptr % 60))
    elseif (ptr < db.warnThreshold) then
        self.text:SetText(string.format("%.1f", ptr))
        if (not self.warned) then
            self.warned = true
            updateDelay = WarningUpdateDelay
            self:setTextColor(db.warnColor)
            if (db.warnSound and LSM) then
                PlaySoundFile(LSM:Fetch("sound", db.warnSoundName)) 
            end
        end
    else
        self.text:SetText(string.format("%d", ptr))
    end
end

function PetTimeToLive:UNIT_PET(event, unitId)
    if (unitId ~= "player") then return end
    if (GetPetTimeRemaining()) then
        self:start()
    else
        self:stop()
    end
end

function PetTimeToLive:mediaUpdate(event, mediaType, key)
    if (mediaType == 'font') then
        if (key == db.font) then
            self:applyFontSettings()
        end
    elseif (mediaType == 'background') then
        if (key == db.bgTexture) then
            self:applyBGSettings()
        end
    elseif (mediaType == 'border') then
        if (key == db.bgBorderTexture) then
            self:applyBGSettings()
        end
    end
end

function PetTimeToLive:applyBGSettings()
    if (not db.bgEnabled) then
        self.mainFrame:SetBackdrop(nil)
        return
    end
    self.bg = self.bg or { insets = {} }
    local bg = self.bg
    if (LSM) then
        bg.bgFile = LSM:Fetch("background", db.bgTexture, true)
        if (not bg.bgFile) then
            bg.bgFile = DefaultBGFile
            LSM.RegisterCallback(self, "LibSharedMedia_Registered", "mediaUpdate")
        end
        bg.edgeFile = LSM:Fetch("border", db.bgBorderTexture, true)
        if (not bg.edgeFile) then
            bg.edgeFile = DefaultEdgeFile
            LSM.RegisterCallback(self, "LibSharedMedia_Registered", "mediaUpdate")
        end
    else
        bg.bgFile = DefaultBGFile
        bg.edgeFile = DefaultEdgeFile
    end
    bg.tile = db.bgTile
    bg.tileSize = db.bgTileSize
    bg.edgeSize = db.bgEdgeSize
    local inset = math.floor(db.bgEdgeSize / 4)
    bg.insets.left = inset
    bg.insets.right = inset
    bg.insets.top = inset
    bg.insets.bottom = inset
    self.mainFrame:SetBackdrop(bg)
    self.mainFrame:SetBackdropColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a)
    self.mainFrame:SetBackdropBorderColor(db.bgBorderColor.r, db.bgBorderColor.g, db.bgBorderColor.b, db.bgBorderColor.a)
end

function PetTimeToLive:applyFontSettings()
    local dbFontPath
    if (LSM) then
        dbFontPath = LSM:Fetch("font", db.font, true)
        if (not dbFontPath) then
            dbFontPath = DefaultFontPath
            LSM.RegisterCallback(self, "LibSharedMedia_Registered", "mediaUpdate")
        end
    else
        dbFontPath = DefaultFontPath
    end
    local fontPath, fontSize, fontOutline = self.text:GetFont()
    fontOutline = fontOutline or ""
    if (dbFontPath ~= fontPath or db.fontSize ~= fontSize or db.fontOutline ~= fontOutline) then
        self.text:SetFont(dbFontPath, db.fontSize, db.fontOutline)
    end
end

function PetTimeToLive:applySettings()
    if (not self:IsEnabled()) then return end
    self.mainFrame:ClearAllPoints()
    self.mainFrame:SetPoint(db.point, UIParent, db.relPoint, db.x, db.y)
    self.mainFrame:SetWidth(db.frameWidth)
    self.mainFrame:SetHeight(db.frameHeight)
    self.mainFrame:SetFrameStrata(db.strata)
    self:applyFontSettings()
    self:applyBGSettings()
    self:toggleLocked(db.locked == true)
end

function PetTimeToLive:createOverlay()
    self.overlay = self.mainFrame:CreateTexture("PetTimeToLiveOverlay", "OVERLAY")
    self.overlay:SetTexture(0, 0.42, 0, 0.42)
    self.overlay:SetAllPoints()

    self.overlayText = self.mainFrame:CreateFontString("PetTimeToLiveOverlayText", "OVERLAY", "GameFontNormal")
    self.overlayText:SetFont(DefaultFontPath, 10, "")
    self.overlayText:SetJustifyH("CENTER")
    self.overlayText:SetPoint("BOTTOM", self.mainFrame, "BOTTOM", 0, 0)
    self.overlayText:SetText("PTTL")
end

function PetTimeToLive:createFrame()
    self.isMoving = false
    self.mainFrame = CreateFrame("Frame", "PetTimeToLiveMainFrame", UIParent)
    self.mainFrame:SetFrameStrata(db.strata)
    self.mainFrame:EnableMouse(false)
    self.mainFrame:SetClampedToScreen()
    self.mainFrame:SetMovable(true)
    self.mainFrame:SetWidth(db.frameWidth)
    self.mainFrame:SetHeight(db.frameHeight)
    self.mainFrame:SetPoint(db.point, UIParent, db.relPoint, db.x, db.y)

    self.text = self.mainFrame:CreateFontString("PetTimeToLiveFrameText", "ARTWORK", "GameFontNormal")
    self.text:SetFont(DefaultFontPath, db.fontSize, db.fontOutline)
    self.text:SetJustifyH("CENTER")
    self.text:SetPoint("CENTER", self.mainFrame, "CENTER", 0, 0)

    self.mainFrame:SetScript("OnMouseDown", function(frame, button)
        if (button == "LeftButton") then
            if (IsControlKeyDown()) then
                self:lock()
                return
            end
            self.mainFrame:StartMoving()
            self.isMoving = true
        elseif (button == "RightButton") then
            self:openConfigDialog()
        end
    end)
    self.mainFrame:SetScript("OnMouseUp", function(frame, button)
        if (self.isMoving and button == "LeftButton") then
            self.mainFrame:StopMovingOrSizing()
            self.isMoving = false
            db.point, _, db.relPoint, db.x, db.y = self.mainFrame:GetPoint()
        end
    end)
    self.mainFrame:SetScript("OnEnter", function(frame)
        GameTooltip:SetOwner(frame)
        GameTooltip:AddLine(AppName)
        GameTooltip:AddLine(L["|cffeda55fDrag|r to move the frame"])
        GameTooltip:AddLine(L["|cffeda55fControl + Left Click|r to lock the frame"])
        GameTooltip:AddLine(L["|cffeda55fRight Click|r to open the configuration window"])
        GameTooltip:Show()
    end)
    self.mainFrame:SetScript("OnLeave", function(frame)
        GameTooltip:Hide()
    end)
    self.mainFrame:SetScript("OnUpdate", function(frame, elapsed)
            lastUpdate = lastUpdate + elapsed
            if (lastUpdate < updateDelay) then return end
            lastUpdate = 0
            self:update()
        end)
end

function PetTimeToLive:lock()
    db.locked = true
    self.mainFrame:EnableMouse(false)
    if (self.overlay) then
        self.overlay:Hide()
        self.overlayText:Hide()
    end
end

function PetTimeToLive:unlock()
    db.locked = false
    if (not self.overlay) then
        self:createOverlay()
    end
    self.mainFrame:EnableMouse(true)
    self.overlay:Show()
    self.overlayText:Show()
    self.mainFrame:Show()
end

function PetTimeToLive:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("PetTimeToLiveDB", defaults)
    db = self.db.profile
    LibStub("LibDualSpec-1.0"):EnhanceDatabase(self.db, AppName)
    self.db.RegisterCallback(self, "OnProfileChanged", "profileChanged")
    self.db.RegisterCallback(self, "OnProfileCopied", "profileChanged")
    self.db.RegisterCallback(self, "OnProfileReset", "profileChanged")
    self:setupOptions()
    if (not self.mainFrame) then
        self:createFrame()
    end
    self:profileChanged()
end

function PetTimeToLive:OnEnable(first)
    self:RegisterEvent("UNIT_PET")
end

function PetTimeToLive:OnDisable()
    if (self.mainFrame) then
        self.mainFrame:Hide()
    end
    self:UnregisterAllEvents()
end

function PetTimeToLive:profileChanged()
    db = self.db.profile
    self:applySettings()
end

function PetTimeToLive:toggleLocked(flag)
    if (flag == nil) then flag = not db.locked end
    if (flag) then
        self:lock()
    else
        self:unlock()
    end
end

